{ stdenv
, dtc
, lzma
, runCommand
, ubootTools
, kernel
, initramfs
, linux-command-line
}:
let
  initramfs_lzma = runCommand "initramfs.lzma" { } ''
    lzma < ${initramfs} > $out;
  '';
  dtb-with-linux-command-line = stdenv.mkDerivation {
    name = "rk3399-gru-kevin-with-linux-command-line.dtb";
    nativeBuildInputs = [ dtc ubootTools ];
    dontUnpack = true;
    buildPhase = ''
      runHook preBuild
      cp ${kernel}/rk3399-gru-kevin.dtb .
      chmod +w rk3399-gru-kevin.dtb
      fdtput -t s -v -p rk3399-gru-kevin.dtb /chosen bootargs '${linux-command-line}'
      runHook posBuild
    '';
    installPhase = ''
      runHook preInstall
      mv rk3399-gru-kevin.dtb $out
      runHook postInstall
    '';
  };
in stdenv.mkDerivation {
  name = "ownerboot-stage1-fit";
  dontUnpack = true;
  nativeBuildInputs = [ dtc ubootTools ];
  buildPhase = ''
    runHook preBuild
    mkdir -p $out
    mkimage \
      -D "-I dts -O dtb -p 2048 -W no-unit_address_vs_reg" \
      -f $dtsPath \
      $out/Image.fit
    runHook postBuild
    '';
  dontInstall = true;
  passAsFile = [ "dts" ];
  dts = ''
      /dts-v1/;
      / {
          description = "kernel image with one or more FDT blobs";
          images {
              kernel {
                  description = "kernel";
                  data = /incbin/("${kernel}/Image.lzma");
                  type = "kernel_noload";
                  arch = "arm64";
                  os = "linux";
                  compression = "lzma";
                  load = <0x06000000>;
                  entry = <0>;
                  hash {
                      algo = "sha1";
                  };
              };
              ramdisk {
                  description = "initramfs";
                  data = /incbin/("${initramfs_lzma}");
                  type = "ramdisk";
                  arch = "arm64";
                  os = "linux";
                  compression = "none";
                  load = <0x02080000>;
                  entry = <0>;
                  hash {
                      algo = "sha1";
                  };
              };
              fdt {
                  description = "rk3399-gru-kevin.dtb";
                  data = /incbin/("${dtb-with-linux-command-line}");
                  type = "flat_dt";
                  arch = "arm64";
                  compression = "none";
                  load = <0x01f00000>;
                  hash {
                      algo = "sha1";
                  };
              };
          };
          configurations {
              default = "conf";
              conf {
                  kernel = "kernel";
                  fdt = "fdt";
                  ramdisk = "ramdisk";
              };
          };
      };
      '';
}
